//=============================================================================
//  MuseScore
//  Music Composition & Notation
//
//  ColorVoices plugin
//
//  Copyright (C)2011 Charles Cave (charlesweb@optusnet.com.au)
//  Copyright (C)2014 Jörn Eichler (joerneichler@gmx.de)
//  Copyright (C)2012-2015 Joachim Schmitz (jojo@schmitz-digital.de)
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2
//  as published by the Free Software Foundation and appearing in
//  the file LICENCE.GPL
//=============================================================================

import QtQuick 2.2
import MuseScore 1.0

MuseScore {
   version:  "1.0"
   description: "This plugin colors the chords and rests of each voice"
   menuPath: "Plugins.Notes.Color Voices"

   property variant colors: [
      "#1259d0", // Voice 1 - Blue    18  89 208
      "#009234", // Voice 2 - Green    0 146  52
      "#c04400", // Voice 3 - Orange 192  68   0
      "#71167a", // Voice 4 - Purple 113  22 122
      "#000000"  // Black
      ]

   function toggleColor(element, color) {
      if (element.color != colors[4])
         element.color = colors[4]
      else
         element.color = color
      }

   function colorVoices(element, voice) {
      if (element.type == Element.REST)
         element.color = colors[voice % 4];
      else if (element.type == Element.CHORD) {
         if (element.stem)
            element.stem.color = colors[voice % 4]
         if (element.hook)
            element.hook.color = colors[voice % 4]
         if (element.beam)
            element.beam.color = colors[voice % 4]
         if (element.stemSlash)
            element.stemSlash.color = colors[voice % 4]
         }
      else if (element.type == Element.NOTE) {
         element.color = colors[voice % 4]
         if (element.accidental)
            element.accidental.color = colors[voice % 4]
         for (var i = 0; i < element.dots.length; i++) {
            if (element.dots[i])
               element.dots[i].color = colors[voice % 4]
            }
         }          
      }

   // Apply the given function to all notes in selection
   // or, if nothing is selected, in the entire score
   function applyToElementsInSelection(func) {
      var cursor = curScore.newCursor()
      cursor.rewind(1)
      var startStaff
      var endStaff
      var endTick
      var fullScore = false
      if (!cursor.segment) { // no selection
         fullScore = true
         startStaff = 0 // start with 1st staff
         endStaff = curScore.nstaves - 1 // and end with last
         }
      else {
         startStaff = cursor.staffIdx
         cursor.rewind(2)
         if (cursor.tick == 0) {
            // this happens when the selection includes
            // the last measure of the score.
            // rewind(2) goes behind the last segment (where
            // there's none) and sets tick=0
            endTick = curScore.lastSegment.tick + 1
            }
         else {
            endTick = cursor.tick
            }
         endStaff = cursor.staffIdx
         }
      console.log(startStaff + " - " + endStaff + " - " + endTick)
      for (var staff = startStaff; staff <= endStaff; staff++) {
         for (var voice = 0; voice < 4; voice++) {
            cursor.rewind(1) // sets voice to 0
            cursor.voice = voice //voice has to be set after goTo
            cursor.staffIdx = staff

            if (fullScore)
               cursor.rewind(0) // if no selection, beginning of score

            while (cursor.segment && (fullScore || cursor.tick < endTick)) {
               if (cursor.element) {
                  if (cursor.element.type == Element.REST)
                     func(cursor.element, voice)
                  else if (cursor.element.type == Element.CHORD) {
                     func(cursor.element, voice)
                     var graceChords = cursor.element.graceNotes;
                     for (var i = 0; i < graceChords.length; i++) {
                        // iterate through all grace chords
                        func(graceChords[i], voice)
                        var notes = graceChords[i].notes
                        for (var j = 0; j < graceChords[i].notes.length; j++)
                           func(graceChords[i].notes[j], voice)
                        }
                     var notes = cursor.element.notes
                     for (var i = 0; i < notes.length; i++) {
                        var note = notes[i]
                        func(note, voice)
                        }
                     } // end if CHORD
                  } // end if element
               cursor.next()
               } // end while cursor
            } // end for loop
         } // end for loop
      }

   onRun: {
      console.log("Hello, Color Voices")
      if (typeof curScore !== 'undefined')
         applyToElementsInSelection(colorVoices)            
      Qt.quit()
      }
}
